CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: scan_value_type.fpp,v 1.2 2014/01/10 19:22:10 fclement Exp $

C     
C     Routines to scan the type of typed values.
C     
C     Value_type ::= Scalar_type | Array_type
C     Array_type ::= Vector_type | Multi_D_type
C     Vector_type ::= "[1" Scalar_type "1]"
C     Multi_D_type ::= '[' Natural p Scalar_type Natural p ']'
C     Scalar_type ::= Simple_type | Tuple_type
C     Tuple_type ::=
C        | '(' Simple_type ", " Simple_type ')'
C        | '(' Simple_type ", " Simple_type ", " Simple_type ')'
C     Simple_type ::= "%S" | "%i" | "%f" | "%bf"
      
#include "pio_constants.h"
      
      integer function scan_binary_type()
C     Scan the binary type (for float values).
      
      implicit none
      
C     Local variables.
      character sbr*16, c
      integer typ
      
C     External function.
      character get_scan_current_char
      
      sbr = 'scan_binary_type'
      
      c = get_scan_current_char()
      
      if (c .eq. 'f') then
         call advance_and_fill_scan_current_char(sbr)
      else
         call error_wrong_typed_value_binary_type(sbr, c)
      endif
      
      call set_binary_value_encoding
      
      if (c .eq. 'f') then
         typ = SFLOAT
      endif
      
      scan_binary_type = typ
      
      return
      end
      
      
      integer function scan_simple_type()
C     Scan simple type.
      
      implicit none
      
C     Local variables.
      character sbr*16, c
      integer typ
      
C     External functions.
      character get_scan_current_char
      integer scan_binary_type
      
      sbr = 'scan_simple_type'
      
      call verify_scan_current_char_and_advance(sbr, '%')
      
      c = get_scan_current_char()
      
      if ((c .eq. 'S') .or. (c .eq. 'i') .or.
     +     (c .eq. 'f') .or. (c .eq. 'b')) then
         call advance_and_fill_scan_current_char(sbr)
      else
         call error_wrong_typed_value_type(sbr, c)
      endif
      
      call set_ascii_value_encoding
C     Value is not binary, except if a 'b' char is detected.
      
      if (c .eq. 'S') then
         typ = SSTRING
      else if (c .eq. 'i') then
         typ = SINT
      else if (c .eq. 'f') then
         typ = SFLOAT
      else if (c .eq. 'b') then
         typ = scan_binary_type()
      endif
      
      scan_simple_type = typ
      
      return
      end
      
      
      integer function scan_tuple_type()
C     Scan tuple type.
      
      implicit none
      
C     Local variables.
      character sbr*15, c
      integer typ, item_typ
      
C     External functions.
      character get_scan_current_char
      integer scan_simple_type
      
      sbr = 'scan_tuple_type'
      
      call verify_scan_current_char_and_advance(sbr, '(')
      typ = scan_simple_type()
      call verify_scan_current_string_and_advance(sbr, ', ', 2)
      
      item_typ = scan_simple_type()
      if (item_typ .ne. typ) then
         call error_heterogeneous_tuple_type(sbr, typ, item_typ)
      endif
      
      c = get_scan_current_char()
      
      if (c .eq. ')') then
         call verify_scan_current_char_and_advance(sbr, ')')
         typ = MAKE_COUPLE_TYPE(typ)
      else
         call verify_scan_current_string_and_advance(sbr, ', ', 2)
         item_typ = scan_simple_type()
         if (item_typ .ne. typ) then
            call error_heterogeneous_tuple_type(sbr, typ, item_typ)
         endif
         call verify_scan_current_char_and_advance(sbr, ')')
         typ = MAKE_TRIPLE_TYPE(typ)
      endif
      
      scan_tuple_type = typ
      
      return
      end
      
      
      integer function scan_scalar_type()
C     Scan scalar type.
      
      implicit none
      
C     Local variables.
      character sbr*16, c
      integer typ
      
C     External functions.
      character get_scan_current_char
      integer scan_simple_type, scan_tuple_type
      
      sbr = 'scan_scalar_type'
      
      c = get_scan_current_char()
      
      if (c .eq. '%') then
         typ = scan_simple_type()
      else if (c .eq. '(') then
         typ = scan_tuple_type()
      else
C     impossible, already checked in scan_value_type
         call error_internal_error(sbr)
      endif
      
      scan_scalar_type = typ
      
      return
      end
      
      
      integer function scan_array_type()
C     Scan array type.
      
      implicit none
      
C     Local variables.
      character sbr*15
      integer p, q, typ
      
C     External functions.
      integer scan_natural, scan_scalar_type
      
      sbr = 'scan_array_type'
      
      call verify_scan_current_char_and_advance(sbr, '[')
      
      p = scan_natural(sbr)
      
      if (p .gt. MAX_ARRAY_DIMENSION) then
         call error_too_many_array_dimensions(sbr, p)
      endif
      
      typ = scan_scalar_type()
      
      q = scan_natural(sbr)
      if (q .ne. p) then
         call error_incoherent_array_markers(sbr, p, q)
      endif
      
      if (p .eq. 1) then
         typ = MAKE_VECTOR_TYPE(typ)
      else
         typ = MAKE_MULTI_D_TYPE(typ)
      endif
      
      call verify_scan_current_char(sbr, ']')
      
      scan_array_type = typ
      
      return
      end
      
      
      integer function scan_value_type(ic)
C     Scan value type.
      
      implicit none
      
C     Input argument.
      integer ic
      
C     Local variables.
      character sbr*15, c
      integer typ
      
C     External functions.
      logical pio_get_report_mode
      character get_scan_current_char
      integer scan_scalar_type, scan_array_type
      
      sbr = 'scan_value_type'
      
      call fill_buffer(sbr, ic)
      c = get_scan_current_char()
      
      if ((c .eq. '%') .or. (c .eq. '(')) then
         typ = scan_scalar_type()
      else if (c .eq. '[') then
         typ = scan_array_type()
      else
         call error_wrong_typed_value_type(sbr, c)
      endif
      
      if (pio_get_report_mode()) then
         write(PIO_REPORT, *) '[Pio/Fortran] type read is ', typ
      endif
      
      scan_value_type = typ
      
      return
      end
