CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: store_utils.fpp,v 1.2 2014/01/10 19:22:10 fclement Exp $

C     
C     Routines to store data into the low-level memory.
C     
      
#include "pio_constants.h"
      
C     
C     Primitives to store data into the memory.
C     
      
      subroutine rand_store(sbr, io, loc, idatum, idx, len)
C     Random storage of data into the memory.
C     Stores at location loc len bytes read from idatum (input data to
C     write) starting from index idx.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*), idatum*(*)
      integer io, loc, idx, len
      
C     Local variables.
      integer end_idx, end_loc
      
      end_idx = idx + (len - 1)
      end_loc = loc + (len - 1)
      
      call verify_rand_store(sbr, io, loc, end_loc)
      
      memory(io)(loc:end_loc) = idatum(idx:end_idx)
      
      call set_memory_clean(io, .False.)
      
      return
      end
      
      
      subroutine seq_store(sbr, io, idatum, idx, len)
C     Sequential storage of data into the memory.
C     Stores at location store_ptr len bytes read from idatum (input
C     data to write) starting from index idx.
C     The store pointer store_ptr is incremented to reflect that len
C     bytes has been stored.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), idatum*(*)
      integer io, idx, len
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store(sbr, io, loc, idatum, idx, len)
      call advance_store_ptr(io, len)
      
      return
      end
      
      
C     
C     Random storing of basic values into the memory.
C     
      
      subroutine rand_store_char(sbr, io, loc, cdatum)
C     Reads one byte from character cdatum and stores it into the memory
C     at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), cdatum
      integer io, loc
      
C     External function.
      logical pio_get_debug_mode
      
      call rand_store(sbr, io, loc, cdatum, 1, 1)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *)
     +        'Character ''', cdatum, ''' stored.'
      endif
      
      return
      end
      
      
      subroutine rand_store_string(sbr, io, loc, sdatum, idx, len)
C     Reads len bytes from string sdatum, starting at index idx, and
C     stores them into the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), sdatum*(*)
      integer io, loc, idx, len
      
C     Local variable.
      integer end_idx
      
C     External function.
      logical pio_get_debug_mode
      
      call rand_store(sbr, io, loc, sdatum, idx, len)
      
      if (pio_get_debug_mode()) then
         end_idx = idx + (len - 1)
         write(PIO_REPORT, *)
     +        'String "', sdatum(idx:end_idx), '" stored.'
      endif
      
      return
      end
      
      
      subroutine rand_store_int(sbr, io, loc, idatum)
C     Reads integer idatum as INT_SIZE bytes and stores them into the
C     memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, idatum
      
C     Local variable.
      character s*INT_SIZE
      
C     External functions.
      logical pio_get_debug_mode
      character int_to_string_bin*INT_SIZE
      
      s = int_to_string_bin(idatum)
      call rand_store(sbr, io, loc, s, 1, INT_SIZE)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *) 'Integer ', idatum, ' stored.'
      endif
      
      return
      end
      
      
      subroutine rand_store_float(sbr, io, loc, fdatum)
C     Reads real fdatum as FLOAT_SIZE bytes and stores them into the
C     memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      FLOAT fdatum
      
C     Local variable.
      character s*FLOAT_SIZE
      
C     External functions.
      logical pio_get_debug_mode
      character float_to_string_bin*FLOAT_SIZE
      
      s = float_to_string_bin(fdatum)
      call rand_store(sbr, io, loc, s, 1, FLOAT_SIZE)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *) 'Float ', fdatum, ' stored.'
      endif
      
      return
      end
      
      
C     
C     Sequential storing of basic values into the memory.
C     
      
      subroutine seq_store_char(sbr, io, cdatum)
C     Reads one byte from character cdatum and stores it into the memory
C     at current store_ptr location.
C     The store pointer store_ptr is incremented to reflect that one
C     byte has been stored.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), cdatum
      integer io
      
C     External function.
      logical pio_get_debug_mode
      
      call seq_store(sbr, io, cdatum, 1, 1)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *) 'Character ''', cdatum, ''' stored.'
      endif
      
      return
      end
      
      
      subroutine seq_store_string(sbr, io, sdatum, idx, len)
C     Reads len bytes from sdatum starting at index idx and stores them
C     into the memory at current store_ptr location.
C     The store pointer store_ptr is incremented to reflect that len
C     bytes has been stored.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), sdatum*MAX_STRING_LENGTH
      integer io, idx, len
      
C     Local variable.
      integer end_idx
      
C     External function.
      logical pio_get_debug_mode
      
      call seq_store(sbr, io, sdatum, idx, len)
      
      if (pio_get_debug_mode()) then
         end_idx = idx + (len - 1)
         write(PIO_REPORT, *)
     +        'String "', sdatum(idx:end_idx), '" stored.'
      endif
      
      return
      end
      
      
      subroutine seq_store_int(sbr, io, idatum)
C     Reads integer idatum as INT_SIZE bytes and stores them into the
C     memory starting at current store_ptr location.
C     The store pointer store_ptr is incremented to reflect that
C     INT_SIZE bytes has been stored.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, idatum
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store_int(sbr, io, loc, idatum)
      call advance_store_ptr(io, INT_SIZE)
      
      return
      end
      
      
      subroutine seq_store_float(sbr, io, fdatum)
C     Reads real fdatum as FLOAT_SIZE bytes and stores them into the
C     memory starting at current store_ptr location.
C     The store pointer store_ptr is incremented to reflect that
C     FLOAT_SIZE bytes has been stored.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      FLOAT fdatum
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store_float(sbr, io, loc, fdatum)
      call advance_store_ptr(io, FLOAT_SIZE)
      
      return
      end
      
      
C     
C     Storing of compound values into the memory (tuples, vectors,
C     matrices).
C     
      
      subroutine rand_store_string_vect
     +     (sbr, io, loc, svdatum, vlen, idx, len)
C     Stores into the memory (starting at loc) the subvector of length
C     len starting at index idx in svdatum.
C     More precisely, rand_store_string_vect reads len string values from
C     the vector svdatum, starting at index idx, as
C     $\sum_{i=idx}^{idx+len-1} vlen(i)$ bytes and stores those bytes into
C     the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), svdatum(*)*(*)
      integer io, vlen(*), idx, len
      
C     Input/output argument.
      integer loc
      
C     Local variable.
      integer i
      
      do i = idx, idx + len - 1
         call rand_store_string(sbr, io, loc, svdatum(i), 1, vlen(i))
         loc = loc + vlen(i)
      enddo
      
      return
      end
      
      
      subroutine rand_store_int_vect(sbr, io, loc, ivdatum, idx, len)
C     Stores into the memory (starting at loc) the subvector of length
C     len starting at index idx in ivdatum.
C     More precisely, rand_store_int_vect reads len integer values from
C     the vector ivdatum, starting at index idx, as len*INT_SIZE bytes
C     and stores those bytes into the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, ivdatum(*), idx, len
      
C     Local variable.
      integer i
      
      do i = idx, idx + len - 1
         call rand_store_int
     +        (sbr, io, loc + (i-idx)*INT_SIZE, ivdatum(i))
      enddo
      
      return
      end
      
      
      subroutine rand_store_float_vect(sbr, io, loc, fvdatum, idx, len)
C     Stores into the memory (starting at loc) the subvector of length
C     len starting at index idx in fvdatum.
C     More precisely, rand_store_float_vect reads len real values from
C     the vector fvdatum, starting at index idx, as len*FLOAT_SIZE bytes
C     and stores those bytes into the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, idx, len
      FLOAT fvdatum(*)
      
C     Local variable.
      integer i
      
      do i = idx, idx + len - 1
         call rand_store_float
     +        (sbr, io, loc + (i-idx)*FLOAT_SIZE, fvdatum(i))
      enddo
      
      return
      end
      
      
      subroutine seq_store_string_vect(sbr, io, svdatum, vlen, idx, len)
C     Stores into the memory (starting at current store_ptr location)
C     the subvector of length len starting into index idx in svdatum.
C     More precisely, seq_store_string_vect reads len string values from
C     the vector svdatum, starting at index idx, as
C     $\sum_{i=idx}^{idx+len-1} vlen(i)$ bytes and stores those bytes into
C     the memory starting at current store_ptr location.
C     The store pointer store_ptr is incremented to reflect that
C     $\sum_{i=idx}^{idx+len-1} vlen(i)$ bytes has been stored.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), svdatum(*)*(*)
      integer io, vlen(*), idx, len
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store_string_vect(sbr, io, loc, svdatum, vlen, idx, len)
      call set_store_ptr(io, loc)
      
      return
      end
      
      
      subroutine seq_store_int_vect(sbr, io, ivdatum, idx, len)
C     Stores into the memory (starting at current store_ptr location)
C     the subvector of length len starting into index idx in ivdatum.
C     More precisely, seq_store_int_vect reads len integer values from
C     the vector ivdatum, starting at index idx, as len*INT_SIZE bytes
C     and stores those bytes into the memory starting at current
C     store_ptr location.
C     The store pointer store_ptr is incremented to reflect that
C     len*INT_SIZE bytes has been stored.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, ivdatum(*), idx, len
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store_int_vect(sbr, io, loc, ivdatum, idx, len)
      call advance_store_ptr(io, len*INT_SIZE)
      
      return
      end
      
      
      subroutine seq_store_float_vect(sbr, io, fvdatum, idx, len)
C     Stores into the memory (starting at current store_ptr location)
C     the subvector of length len starting into index idx in fvdatum.
C     More precisely, seq_store_float_vect reads len real values from
C     the vector fvdatum, starting at index idx, as len*FLOAT_SIZE bytes
C     and stores those bytes into the memory starting at current
C     store_ptr location.
C     The store pointer store_ptr is incremented to reflect that
C     len*FLOAT_SIZE bytes has been stored.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      FLOAT fvdatum(*)
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_store_ptr
      
      loc = get_store_ptr(io)
      
      call rand_store_float_vect(sbr, io, loc, fvdatum, idx, len)
      call advance_store_ptr(io, len*FLOAT_SIZE)
      
      return
      end
